Package: fingerprint
Version: 3.5.10
Date: 2026-04-03
Title: Functions to Operate on Binary Fingerprint Data
Authors@R: c(
    person(given = "Rajarshi",
           family = "Guha",
           role = c("aut"),
           email = "rajarshi.guha@gmail.com"),
    person(given = "Zach",
           family = "Charlop-Powers",
           role = c("cre"),
           email = "zach.charlop.powers@gmail.com"))
Author: Rajarshi Guha [aut],
  Zach Charlop-Powers [cre]
Maintainer: Zach Charlop-Powers <zach.charlop.powers@gmail.com>
BugReports: https://github.com/CDK-R/cdkr/issues
Description: Functions to manipulate binary fingerprints
 of arbitrary length. A fingerprint is represented by an object of S4 class 'fingerprint'
 which is internally represented a vector of integers, such
 that each element represents the position in the fingerprint that is set to 1.
 The bitwise logical functions in R are overridden so that they can be used directly
 with 'fingerprint' objects. A number of distance metrics are also
 available (many contributed by Michael Fadock). Fingerprints
 can be converted to Euclidean vectors (i.e., points on the unit hypersphere) and
 can also be folded using OR.  Arbitrary fingerprint formats can be handled via line
 handlers. Currently handlers are provided for CDK, MOE and BCI fingerprint data.
License: GPL | file LICENSE
Depends: methods
LazyLoad: yes
Suggests: RUnit, testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-04-03 16:52:07 UTC; zcpowers
Repository: CRAN
Date/Publication: 2026-04-04 05:11:07 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-08 01:03:23 UTC; unix
Archs: fingerprint.so.dSYM

importFrom("methods", "new")
exportClasses("fingerprint")
exportClasses("featvec")
exportClasses("feature")
exportMethods("fold", "euc.vector", "distance",
              "random.fingerprint", "as.character",
              "length", 
              "feature", "count", 'feature<-', 'count<-')
export("fp.sim.matrix", "fp.to.matrix", "fp.factor.matrix",
       "fp.read.to.matrix", "fp.read","shannon",
       ##"featvec.to.binaryfp",
       "moe.lf", "bci.lf", "cdk.lf", "ecfp.lf", "fps.lf", "jchem.binary.lf",
       "bit.spectrum", "balance", "bit.importance")
useDynLib(fingerprint,.registration = TRUE)

Fingerprint -- Document Fingerprint Generator
---------------------------------------------

Fingerprint is a signature of the document. In particular, it is a representative subset of hash values from the set of all hash values of a document. For more detail, please consider taking a look at [Winnowing: Local Algorithms for Document Fingerprinting](http://theory.stanford.edu/~aiken/publications/papers/sigmod03.pdf) *(specifically Figure 2)*.

Fingerprint Module Installation
-------------------------------

The recommended way to install the `fingerprint` module is to simply use `pip`:

```console
$ pip install fingerprint
```
Fingerprint officially supports Python >= 3.0.

How to use fingerprint?
-----------------------
```pycon
>>> from fingerprint import Fingerprint
>>> fprint = Fingerprint(kgram_len=4, window_len=5, base=10, modulo=1000)
>>> fprint.generate(str="adorunrunrunadorunrun")
>>> fprint.generate(fpath="../CHANGES.txt")
```

The default values for the parameters are
```python
kgram_len = 50
window_len = 100
base = 101
modulo = sys.maxint
```

===========
fingerprint
===========

    * **What is it?**
        
        "fingerprint" module generates fingerprints of a document.
    
    * **Fingerprint!!!! :(, What is that?**
        
        Fingerprint is like the signature of the document. More specifically, in our context, it is the subset of hash values calculated from the document.

    * **Okay i now know about it a bit, tell me how it calculates fingerprints?**
        
        Generation of fingerprints of a document is a three stage process;

        - (1st phase) generates the k-grams from the standard string 
        - (2nd phase) generates the hash values for each k-gram using rolling hash function 
        - (3rd phase) generates the fingerprints from the hash values using winnowing

    * **How can i install it on my machine?** 
    
        You can install it in basically two ways;

        - using source
            1. git clone git@github.com:kailashbuki/fingerprint.git
            2. cd fingerprint
            3. sudo python setup.py install
    
        - using pip
            1. sudo pip install fingerprint
    
    * **Hmm! ... How can i use it?**
    
        It's plain simple. Here's an example for you;

::

                from fingerprint.fingerprintgenerator import file_content_refiner, FingerprintGenerator
              
                # You could get the standard string from a document as;
                s = file_content_refiner("path/to/file")
                # OR you could directly pass the standard string if you have
                s = "some sample string"
                
                fpg = FingerprintGenerator(input_string=s)
                fpg.generate_fingerprints()
                print fpg.fingerprints
    
    >>Feel free to contact at kailash<DOT>buki<AT>gmail<DOT>com (kailash.buki@gmail.com)<<

jchem.binary.lf <- function(line) {
  molid <- strsplit(line, "\t")[[1]][1]
  bitpos <- .Call("parse_jchem_binary", as.character(line), as.integer(nchar(line)) )
  if (is.null(bitpos)) return(NULL)
  list(molid, bitpos+1, list()) ## we add 1, since C does bit positions from 0  
}

fps.lf <- function(line) {
  toks <- strsplit(line, "\\s")[[1]];
  title <- paste(toks[2:length(toks)], collapse=' ', sep='')
  bitpos <- .Call("parse_hex", as.character(toks[1]), as.integer(nchar(toks[1])))
  if (is.null(bitpos)) return(NULL)
  if (length(toks) > 2) {
    misc <- list(toks[-c(1,2)])
  } else { misc <- list() }
  list(title, bitpos+1, misc) ## we add 1, since C does bit positions from 0
}

cdk.lf <- function(line) {
  p <- regexpr("{([0-9,\\s]*)}",line,perl=T)
  s <- gsub(',','',substr(line, p+1, p+attr(p,"match.length")-2))
  s <- lapply( strsplit(s,' '), as.numeric )
  molid <- gsub("\\s+","", strsplit(line, "\\{")[[1]][1])
  list(molid, s[[1]], list())
}

moe.lf <- function(line) {
  p <- regexpr("\"([0-9\\s]*)\"",line, perl=T)
  s <- substr(line, p+1, p+attr(p,"match.length")-2)
  s <- lapply( strsplit(s,' '), as.numeric )
  list(NA, s[[1]], list())
}

bci.lf <- function(line) {
  tokens <- strsplit(line, '\\s')[[1]]
  name <- tokens[1]
  tokens <- tokens[-c(1, length(tokens), length(tokens)-1)]
  list(name, as.numeric(tokens), list())
}

ecfp.lf <- function(line) {
  tokens <- strsplit(line, '\\s')[[1]]
  name <- tokens[1]
  tokens <- tokens[-1]
  list(name, tokens, list())
}

## TODO we should be iterating over lines and not reading
## them all in
fp.read <- function(f='fingerprint.txt', size=1024, lf=cdk.lf, header=FALSE, binary=TRUE) {
  lf.name <- deparse(substitute(lf))
  
  provider <- lf.name
  
  fplist <- list()
  fcon <- file(description=f,open='r')
  lines = readLines(fcon,n=-1)
  if (header && lf.name != 'fps.lf') lines = lines[-1]
  if (lf.name == 'fps.lf') {
    binary <- TRUE
    size <- NULL
    ## process the header block
    nheaderline = 0
    for (line in lines) {
      if (substr(line,1,1) != '#') break
      nheaderline <- nheaderline + 1
      if (nheaderline == 1 && length(grep("#FPS1", line)) != 1) stop("Invalid FPS format")
      if (length(grep("#num_bits", line)) == 1) size <- as.numeric(strsplit(line, '=')[[1]][2])
      if (length(grep("#software", line)) == 1) provider <- as.character(strsplit(line, '=')[[1]][2])      
    }
    lines <- lines[ (nheaderline+1):length(lines) ]
    if (is.null(size)) { # num_bit
      size <- nchar(strsplit(line, '\\s')[[1]][1]) * 4
    }
  }
  c = 1
  for (line in lines) {
    dat <- lf(line)
    if (is.null(dat)) {
      warning(sprintf("Couldn't parse: %s", line))
      next
    }
    if (is.na(dat[[1]])) name <- ""
    else name <- dat[[1]]

    misc <- dat[[3]] ## usually empty
    if (binary) {
      fplist[[c]] <- new("fingerprint",
                         nbit=size,
                         bits=as.numeric(dat[[2]]),
                         folded=FALSE,
                         provider=provider,
                         name=name,
                         misc=misc)
    } else {
      ## convert the features to 'feature' objects
      feats <- lapply(dat[[2]], function(x) new("feature", feature=x))
      fplist[[c]] <- new("featvec",
                         features=feats,
                         provider=provider,
                         name=name,
                         misc=misc)
    }
    c <- c+1
      }
  close(fcon)
  fplist
}

## Need to supply the length of the bit string since fp.read does
## not provide that information
fp.read.to.matrix <- function(f='fingerprint.txt', size=1024, lf=cdk.lf, header=FALSE) {
  fplist <- fp.read(f, size, lf, header)
  fpmat <- fp.to.matrix(fplist)
  fpmat
}

fp.sim.matrix <- function(fplist, fplist2=NULL, method='tanimoto') {

  sim <- NA
  
  if (!is.null(fplist2)) {
      sim <- do.call('rbind', lapply(fplist,
                                     function(fp) unlist(lapply(fplist2,
                                                                function(x) distance(x,fp, method=method)))))
      ##diag(sim) <- 1.0
      return(sim)
  }

  if (method == 'dice') {
    sim <- .dice.sim.mat(fplist)
  } else if (method == 'tanimoto') {
    sim <- .tanimoto.sim.mat(fplist)
  } else {
    sim <- matrix(0,nrow=length(fplist), ncol=length(fplist))
    for (i in 1:(length(fplist)-1)) {
      v <- unlist(lapply( fplist[(i+1):length(fplist)], distance, fp2=fplist[[i]], method=method))
      sim[i,(i+1):length(fplist)] <- v
      sim[(i+1):length(fplist),i] <- v
    }
  }
  diag(sim) <- 1.0
  return(sim)
}

## Takes the fingerprints, P bits,  for a set of N molecules supplied as
## a list structure and creates an N x P matrix
fp.to.matrix <- function( fplist ) {
  size <- fplist[[1]]@nbit
  m <- matrix(0, nrow=length(fplist), ncol=size)
  cnt <- 1
  for ( i in fplist ) {
    m[cnt,i@bits] <- 1
    cnt <- cnt + 1
  }
  m
}

fp.factor.matrix <- function( fplist ) {
  size <- fplist[[1]]@nbit
  m <- data.frame(fp.to.matrix(fplist))
  m[] <- lapply(m, factor, levels=0:1)
  m
}

.dice.sim.mat <- function(fplist) {
  m <- fp.to.matrix(fplist)
  mat<-m%*%t(m)
  len<-length(m[,1])
  s<-mat.or.vec(len,len)
  rs<-rowSums(m) #since its is binary just add the row values.

  for (i in 1:(len-1)) {
    for (j in (i+1):len) {
      s[i,j]=(2*(mat[i,j])/(rs[i]+rs[j]))
      s[j,i]=s[i,j]
    }
  }
  diag(s) <- 1.0  
  return(s)
}

.tanimoto.sim.mat <- function(fplist){
  m <- fp.to.matrix(fplist)
  mat<-m%*%t(m)
  len<-length(m[,1])
  s<-mat.or.vec(len,len)
  
  ret <-  .C("m_tanimoto", as.double(mat), as.integer(len), as.double(s),
             PACKAGE="fingerprint")
  ret <- matrix(ret[[3]], nrow=len, ncol=len, byrow=TRUE)
  return(ret)
  
  ## for (i in 1:len){
  ##   for (j in 1:len){
  ##     s[i,j]<- mat[i,j]/(mat[i,i]+mat[j,j]-mat[i,j]) # Formula for Tanimoto Calculation
  ##   }
  ## }
  ## return(s)
}

test.new.fp <- function()
{
  fp <- new("fingerprint", bits=c(1,2,3,4), nbit=8, provider='rg',name='foo')
  checkTrue(!is.null(fp))
}

test.distance1 <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fp2 <- new("fingerprint",
             bits=c(5,6,7,8), nbit=8)
  d <- distance(fp1,fp2)
  checkEquals(d, 0)
}

test.distance2 <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fp2 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  d <- distance(fp1,fp2)
  checkEquals(d, 1)
}

test.and1 <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fp2 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fpnew <- fp1 & fp2
  bits <- fpnew@bits
  checkTrue( all(bits == c(1,2,3,4)))
}
test.and2 <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fp2 <- new("fingerprint",
             bits=c(5,6,7,8), nbit=8)
  fpnew <- fp1 & fp2
  bits <- fpnew@bits
  checkEquals(length(bits),0)
}

test.or1 <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fp2 <- new("fingerprint",
             bits=c(5,6,7,8), nbit=8)
  fpnew <- fp1 | fp2
  bits <- fpnew@bits
  checkTrue(all(bits == c(1,2,3,4,5,6,7,8)))
}
test.or2 <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fp2 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fpnew <- fp1 | fp2
  bits <- fpnew@bits
  checkTrue(all(bits == c(1,2,3,4)))
}

test.not <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  nfp1 <- !fp1
  checkTrue(all(nfp1@bits == c(5,6,7,8)))
  checkTrue(all(fp1@bits == (!nfp1)@bits))
}

test.xor1 <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fp2 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fpnew <- xor(fp1,fp2)
  bits <- fpnew@bits
  checkEquals(length(bits),0)
}
test.xor2 <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  fp2 <- new("fingerprint",
             bits=c(5,6,7,8), nbit=8)
  fpnew <- xor(fp1,fp2)
  bits <- fpnew@bits
  checkEquals(length(bits),8)
  checkTrue(all(bits == c(1,2,3,4,5,6,7,8)))
}

test.fold1 <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4), nbit=8)
  nfp <- fold(fp1)
  checkTrue(all(nfp@bits == c(1,2,3,4)))
}

test.fold2 <- function() {
  fp1 <- new("fingerprint",
             bits=c(1,2,3,4,8), nbit=8)
  nfp <- fold(fp1)
  checkTrue(all(nfp@bits == c(1,2,3,4)))
}

test.fp.to.matrix <- function() {
    fp1 <- new("fingerprint", bits=c(1,2,3,4), nbit=8)
    fp2 <- new("fingerprint", bits=c(5,6,7,8), nbit=8)
    fp3 <- new("fingerprint", bits=c(1,2,3,5,6,7,8), nbit=8)
    m1 <- fp.to.matrix(list(fp1,fp2,fp3))
    m2 <- rbind(c(1,1,1,1,0,0,0,0),
                c(0,0,0,0,1,1,1,1),
                c(1,1,1,0,1,1,1,1))
    checkTrue(all(m1 == m2))
}

test.tversky.1 <- function() {
  fp1 <- new("fingerprint", bits=c(1,2,3,4), nbit=8)
  fp2 <- new("fingerprint", bits=c(1,2,3,4), nbit=8)
  s <- distance(fp1, fp2, "tversky", a=1,b=1)
  checkEquals(1.0, s)
}
test.tversky.2 <- function() {
  fp1 <- new("fingerprint", bits=c(5,6,7,8), nbit=8)
  fp2 <- new("fingerprint", bits=c(1,2,3,4), nbit=8)
  s <- distance(fp1, fp2, "tversky", a=1,b=1)
  checkEquals(0.0, s)
}
test.tversky.3 <- function() {
  fp1 <- new("fingerprint", bits=c(4,6,7,8), nbit=8)
  fp2 <- new("fingerprint", bits=c(1,2,3,4), nbit=8)
  stv <- distance(fp1, fp2, "tversky", a=1,b=1)
  sta <- distance(fp1, fp2)  
  checkEquals(stv, sta)
}
test.tversky.4 <- function() {
  fp1 <- new("fingerprint", bits=c(4,6,7,8), nbit=8)
  fp2 <- new("fingerprint", bits=c(1,2,3,4), nbit=8)
  stv <- distance(fp1, fp2, "tversky", a=0.5,b=0.5)
  std <- distance(fp1, fp2, "dice")  
  checkEquals(stv, std)
}

test.fp.sim.matrix <- function() {
    fp1 <- new("fingerprint", bits=c(1,2,3,4), nbit=8)
    fp2 <- new("fingerprint", bits=c(5,6,7,8), nbit=8)
    fp3 <- new("fingerprint", bits=c(1,2,3,5,6,7,8), nbit=8)
    fpl <- list(fp1,fp2,fp3)
    sm <- round(fp.sim.matrix(fpl),2)
    am <- rbind(c(1,0,0.38),
                c(0,1,0.57),
                c(0.38,0.57,1))
    checkTrue(all(sm == am))
}

test.fp.balance <- function() {
  fp1 <- new("fingerprint", bits=c(1,2,3), nbit=6)  
  fp2 <- balance(fp1)
  checkTrue(12 == length(fp2))
  checkEquals(c(1,2,3,10,11,12), fp2@bits)
}

test.fps.reader <- function() {
  data.file <- file.path(system.file("unitTests", "bits.fps", package="fingerprint"))
  fps <- fp.read(data.file, lf=fps.lf)
  checkEquals(323, length(fps))

  ## OK, we need to pull in the bit positions Andrew specified
  for (i in seq_along(fps)) {
    expected <- sort(as.numeric(strsplit(fps[[i]]@misc[[1]],",")[[1]])+1)
    observed <- sort(fps[[i]]@bits)
    checkEquals(expected, observed, msg = sprintf("%s had a mismatch in bit positions", fps[[i]]@name))
  }
}

#######################################
##
## Feature vector tests
##
#######################################
test.feature <- function() {
  f1 <- new('feature', feature='F1')
  checkEquals(1, f1@count)

  f2 <- new('feature', feature='F2', count=as.integer(12))
  checkEquals(12, f2@count)
}

test.feature.c <- function() {
  f1 <- new('feature', feature='F1', count=as.integer(2))
  f2 <- new('feature', feature='F2', count=as.integer(3))
  fl <- c(f1, f2)
  checkEquals(2, length(fl))
  checkEquals("list", class(fl))
  checkTrue(identical(f1, fl[[1]]))
  checkTrue(identical(f2, fl[[2]]))  
}

test.feature.fp <- function() {
  feats <- sapply(letters[1:10], function(x) new('feature', feature=x, count=as.integer(1)))
  fv <- new('featvec', features=feats)
  checkEquals(10, length(fv))
}

test.feature.dist1 <- function() {
  f1 <- sapply(letters[1:10], function(x) new('feature', feature=x, count=as.integer(1)))
  f2 <- sapply(letters[1:10], function(x) new('feature', feature=x, count=as.integer(1)))
  fv1 <- new('featvec', features=f1)
  fv2 <- new('featvec', features=f2)
  d <- distance(fv1, fv2, method='tanimoto')
  checkEquals(1, d)
}
test.feature.dist2 <- function() {
  f1 <- sapply(letters[1:10], function(x) new('feature', feature=x, count=as.integer(1)))
  f2 <- sapply(letters[11:20], function(x) new('feature', feature=x, count=as.integer(1)))
  fv1 <- new('featvec', features=f1)
  fv2 <- new('featvec', features=f2)
  d <- distance(fv1, fv2, method='tanimoto')
  checkEquals(0, d)
}

test.featvec.read <- function() {
  data.file <- file.path(system.file("unitTests", "test.ecfp", package="fingerprint"))
  fps <- fp.read(data.file, lf=ecfp.lf, binary=FALSE)
  checkEquals(10, length(fps))

  lengths <- c(58L, 38L, 43L, 66L, 62L, 66L, 65L, 44L, 66L, 61L)
  ol <- sapply(fps, length)
  checkTrue(identical(lengths, ol))
}

tester.getters.setters <- function() {
  f <- new("feature", feature='ABCD', count=as.integer(1))
  checkEquals("ABCD", feature(f))
  checkEquals(1, count(f))

  feature(f) <- 'UXYZ'
  count(f) <- 10
  checkEquals("UXYZ", feature(f))
  checkEquals(10, count(f))
}

setClass("fingerprint",
         representation(bits="numeric",
                        nbit="numeric",
                        folded="logical",
                        provider="character",
                        name="character",
                        misc="list"),
         validity=function(object) {
           if (any(object@bits > object@nbit))
             return("Bit positions were greater than the specified bit length")
           else return(TRUE)           
         },
         prototype(bits=c(),
                   nbit=0,
                   folded=FALSE,
                   provider="",
                   name="",
                   misc=list()))

#setGeneric("show", function(object) standardGeneric("show"))
setMethod("show", "fingerprint",
          function(object) {
            cat("Fingerprint object\n")
            cat(" name = ", object@name, "\n")
            cat(" length = ", object@nbit, "\n")
            cat(" folded = ", object@folded, "\n")
            cat(" source = ", object@provider, "\n")
            cat(" bits on = ", paste(sort(object@bits), collapse=' '), "\n")
          })


setMethod('as.character', "fingerprint",
          function(x) {
            s <- numeric(x@nbit)
            s[x@bits] <- 1
            paste(s,sep='',collapse='')
          })

setMethod("length", "fingerprint",
          function(x) {
            x@nbit
          })

parseCall <- function (obj) 
{
    if (!inherits(obj, "call")) {
        stop("Must supply a 'call' object")
    }
    srep <- deparse(obj)
    if (length(srep) > 1) 
        srep <- paste(srep, sep = "", collapse = "")
    fname <- unlist(strsplit(srep, "\\("))[1]
    func <- unlist(strsplit(srep, paste(fname, "\\(", sep = "")))[2]
    func <- unlist(strsplit(func, ""))
    func <- paste(func[-length(func)], sep = "", collapse = "")
    func <- unlist(strsplit(func, ","))
    vals <- list()
    nms <- c()
    cnt <- 1
    for (args in func) {
        arg <- unlist(strsplit(args, "="))[1]
        val <- unlist(strsplit(args, "="))[2]
        arg <- gsub(" ", "", arg)
        val <- gsub(" ", "", val)
        vals[[cnt]] <- val
        nms[cnt] <- arg
        cnt <- cnt + 1
    }
    names(vals) <- nms
    vals
}

setGeneric("fold", function(fp) standardGeneric("fold"))
setMethod("fold", "fingerprint",
          function(fp) {
            size <- fp@nbit
            if (size %% 2 != 0) {
              stop('Need to supply a fingerprint of even numbered length')
            }
            bfp <- rep(FALSE, size)
            bfp[fp@bits] <- TRUE

            subfplen <- size/2
            
            b1 <- which(bfp[1:subfplen])
            b2 <- which(bfp[(subfplen+1):size])
            
            subfp1 <- new("fingerprint",
                          nbit=subfplen,
                          bits=b1,
                          provider="R");
            
            subfp2 <- new("fingerprint",
                          nbit=subfplen,
                          bits=b2,
                          provider="R")
            foldedfp <- subfp1 | subfp2
            foldedfp@folded <- TRUE
            return(foldedfp)
          })

setGeneric("euc.vector", function(fp) standardGeneric("euc.vector"))
setMethod("euc.vector", "fingerprint",
          function(fp) {
            coord <- rep(0,length(fp))
            coord[fp@bits] <- 1.0 / sqrt(length(fp))
            coord
          })


setGeneric("distance", function(fp1,fp2,method,a,b) standardGeneric("distance"))
setMethod("distance", c("featvec", "featvec", "missing", "missing", "missing"),
          function(fp1, fp2) {
            distance(fp1, fp2, "tanimoto" )
          })
setMethod("distance", c("featvec", "featvec", "character", "missing", "missing"),
          function(fp1, fp2, method=c("tanimoto", "dice", "robust")) {
            method <- match.arg(method)
            n1 <- length(fp1)
            n2 <- length(fp2)
            ## extract the feature strings, ignoring counts for now
            f1 <- sapply(fp1@features, function(x) x@feature)
            f2 <- sapply(fp2@features, function(x) x@feature)
            
            n12 <- length(intersect(f1,f2))
            if (method == 'tanimoto') {
              return(n12/(n1+n2-n12))
            } else if (method == "robust") {
              return(0.5 + 0.5 * n12 * n12 / (n1*n2))
            } else if (method == "dice") {
              return(2.0 * n12 / (n1+n2))
            }
          })

setMethod("distance", c("fingerprint", "fingerprint", "missing", "missing", "missing"),
          function(fp1,fp2) {
            distance(fp1,fp2,"tanimoto")
          })

setMethod("distance", c("fingerprint", "fingerprint", "character", "numeric", "numeric"),
          function(fp1, fp2, method="tversky", a, b) {
            if (!is.null(method) && !is.na(method) && method != "tversky") distance(fp1, fp2, method)
            if ( length(fp1) != length(fp2))
              stop("Fingerprints must of the same bit length")
            if (a < 0 || b < 0) stop("a and b must be positive")

            tmp <- fp1 & fp2
            xiy <- length(tmp@bits)

            tmp <- fp1 | fp2
            xuy <- length(tmp@bits)

            x <- length(fp1@bits)
            y <- length(fp2@bits)
            return( xiy / (a*x + b*y + (1-a-b)*xiy ) )
          })
setMethod("distance", c("fingerprint", "fingerprint", "character", "missing", "missing"),
          function(fp1,fp2, method=c('tanimoto', 'euclidean', 'mt',
                              'simple', 'jaccard', 'dice',
                              'russelrao', 'rodgerstanimoto','cosine',
                              'achiai', 'carbo', 'baroniurbanibuser',
                              'kulczynski2',
                              
                              'hamming', 'meanHamming', 'soergel',
                              'patternDifference', 'variance', 'size', 'shape',

                              'hamann', 'yule', 'pearson', 'dispersion',
                              'mcconnaughey', 'stiles',

                              'simpson', 'petke',
                              'stanimoto', 'seuclidean'
                              )) {

            if (method == 'tversky')
              stop("If Tversky metric is desired, must specify a and b")
            
            if ( length(fp1) != length(fp2))
              stop("Fingerprints must of the same bit length")
            
            method <- match.arg(method)
            n <- length(fp1)

            if (method == 'tanimoto') {
              f1 <- numeric(n)
              f2 <- numeric(n)
              f1[fp1@bits] <- 1
              f2[fp2@bits] <- 1
              sim <- 0.0
              ret <-  .C("fpdistance", as.double(f1), as.double(f2),
                         as.integer(n), as.integer(1),
                         as.double(sim),
                         PACKAGE="fingerprint")
              return (ret[[5]])
            } else if (method == 'euclidean') {
              f1 <- numeric(n)
              f2 <- numeric(n)
              f1[fp1@bits] <- 1
              f2[fp2@bits] <- 1
              sim <- 0.0
              ret <-  .C("fpdistance", as.double(f1), as.double(f2),
                         as.integer(n), as.integer(2),
                         as.double(sim),
                         PACKAGE="fingerprint")
              return (ret[[5]])
            }

            size <- n

            ## in A & B
            tmp <- fp1 & fp2
            c <- length(tmp@bits)

            ## in A not in B
            tmp <- (fp1 | fp2) & !fp2
            a <- length(tmp@bits)

            ## in B not in A
            tmp <- (fp1 | fp2) & !fp1
            b <- length(tmp@bits)

            ## not in A, not in B
            tmp <- !(fp1 | fp2)
            d <- length(tmp@bits)

            dist <- NULL

            ## Simlarity
            if (method == 'stanimoto') {
              dist <- c / (a+b+c)
            } else if (method == 'seuclidean') {
              dist <- sqrt((d+c) / (a+b+c+d))
            } else if (method == 'dice') {
              dist <- c / (.5*a + .5*b + c)
            } else if (method == 'mt') {
              t1 <- c/(size-d)
              t0 <- d/(size-c)
              phat <- ((size-d) + c)/(2*size)
              dist <- (2-phat)*t1/3 + (1+phat)*t0/3
            } else if (method == 'simple') {
              dist <- (c+d)/n
            } else if (method == 'jaccard') {
              dist <- c/(a+b+c)
            } else if (method == 'russelrao') {
              dist <- c/size
            } else if (method == 'rodgerstanimoto') {
              dist <- (c+d)/(2*a+2*b+c+d)
            } else if (method == 'cosine' || method == 'achiai' || method == 'carbo') {
              dist <- c/sqrt((a+c)*(b+c))
            } else if (method == 'baroniurbanibuser') {
              dist <- (sqrt(c*d)+c)/(sqrt(c*d)+a+b+c)
            } else if (method == 'kulczynski2') {
              dist <- .5*(c/(a+c)+c/(b+c))              
            }
            ## Dissimilarity
            else if (method == 'hamming') {
              dist <- a+b
            } else if (method == 'meanHamming') {
              dist <- (a+b)/(a+b+c+d)
            }else if (method == 'soergel') {
              dist <- (a+b)/(a+b+c)
            } else if (method == 'patternDifference') {
              dist <- (a*b)/(a+b+c+d)^2
            } else if (method == 'variance') {
              dist <- (a+b)/(4*n)
            } else if (method == 'size') {
              dist <-  (a-b)^2/n^2
            } else if (method == 'shape') {
              dist <- (a+b)/n-((a-b)/(n))^2
            }

            ## Composite
            else if (method == 'hamann') {
              dist <- (c+d-a-b)/(a+b+c+d)
            } else if (method == 'yule') {
              dist <-  (c*d-a*b)/(c*d+a*b)
            } else if (method == 'pearson') {
              dist <- (c*d-a*b)/sqrt((a+c)*(b+c)*(a+d)*(b+d))
            } else if (method == 'dispersion') {
              dist <- (c*d-a*b)/n^2
            } else if (method == 'mcconnaughey') {
              dist <- (c^2-a*b)/((a+c)*(b+c))
            } else if (method == 'stiles') {
              dist <- log10(n*(abs(c*d-a*b)-n/2)^2/((a+c)*(b+c)*(a+d)*(b+d)))
            }

            ## Asymmetric
            else if (method == 'simpson') {
              dist <- c/min((a+c),(b+c))
            } else if (method == 'petke') {
              dist <- c/max((a+c),(b+c))
            }
            
            dist
          })

setGeneric("random.fingerprint",
           function(nbit, on) standardGeneric("random.fingerprint"))
setMethod("random.fingerprint", c("numeric", "numeric"),
          function(nbit, on) {
            if (nbit <= 0) stop("Bit length must be positive integer")
            if (on <= 0) stop("Number of bits to be set to 1 must be positive integer")            
            bits <- sample(1:nbit, size=on)
            new("fingerprint", nbit=nbit, bits=bits, provider="R", folded=FALSE)
          })

## Define a feature and its count
setClass("feature",
         contains = 'integer',
         representation(feature='character',
                        count='integer'),
         validity=function(object) {
           if (is.na(object@feature) || is.null(object@feature)) return("feature must be a string")
           if (object@count < 0) return("count must be zero or a positive integer")
           return(TRUE)
         },
         prototype(feature='', count=as.integer(1))
         )
setMethod('show', 'feature',
          function(object) {
            cat(sprintf('%s:%d', object@feature, object@count), '\n')
          })
setMethod('as.character', signature(x='feature'), function(x) sprintf("%s:%d", x@feature, x@count))
setMethod('c', signature(x='feature'), function(x, ..., recursive=FALSE) {
  elems <- list(x, ...)
  ret <- list()
  for (i in seq_along(elems)) {
    ret[[i]] <- new("feature", feature=elems[[i]]@feature, count=as.integer(elems[[i]]@count))
  }
  return(ret)
})

## getters/setters
setGeneric("feature", function(object) standardGeneric("feature"))
setMethod("feature", "feature", function(object) object@feature)
setGeneric("feature<-", function(this, value) standardGeneric("feature<-"))
setReplaceMethod("feature", signature=signature("feature", "character"),
                 function(this, value) {
                   this@feature <- value
                   this
                 })

setGeneric("count", function(object) standardGeneric("count"))
setMethod("count", "feature", function(object) object@count)
setGeneric("count<-", function(this, value) standardGeneric("count<-"))
setReplaceMethod("count", signature=signature("feature", "numeric"),
                 function(this, value) {
                   this@count <- as.integer(value)
                   this
})

setMethod("&", c("fingerprint", "fingerprint"),
          function(e1, e2) {
            if (e1@nbit != e2@nbit)
              stop("fp1 & fp2 must of the same bit length")
            
            andbits <- intersect(e1@bits, e2@bits)
            new("fingerprint",
                bits=andbits,
                nbit=e1@nbit,
                provider="R")
          })

setMethod("|", c("fingerprint", "fingerprint"),
          function(e1, e2) {
            if (e1@nbit != e2@nbit)
              stop("fp1 & fp2 must of the same bit length")
            
            orbits <- union(e1@bits, e2@bits)
            new("fingerprint",
                bits=orbits,
                nbit=e1@nbit,
                provider="R")
          })

setMethod("!", c("fingerprint"),
          function(x) {
            bs <- 1:(x@nbit)
            if (length(x@bits) > 0) b <- bs[ -x@bits ]
            else b <- bs
            ret <- new("fingerprint",
                       bits=b,
                       nbit=x@nbit,
                       provider="R")
            return(ret)
          })

setMethod("xor", c("fingerprint", "fingerprint"),
          function(x,y) {
            if (x@nbit != y@nbit)
              stop("e1 & e2 must of the same bit length")

            tmp1 <- rep(FALSE, x@nbit)
            tmp2 <- rep(FALSE, y@nbit)
            tmp1[x@bits] <- TRUE
            tmp2[y@bits] <- TRUE
            tmp3 <- xor(tmp1,tmp2)
            xorbits <- which(tmp3)
            
            new("fingerprint",
                bits=xorbits,
                nbit=x@nbit,
                provider="R")
          })

## A feature fingerprint will be a vector of feature objects
setClass("featvec",
         representation(features="list",
                        provider="character",
                        name="character",
                        misc="list"),
         validity=function(object) {
           ## features must be a list of feature objects
           klasses <- unique(sapply(object@features, class))
           if (length(klasses) != 1 || klasses != 'feature')
             return("Must supply a list of 'feature' objects")
           iss4s <- sapply(object@features, isS4)
           if (!all(iss4s))
             return("Must supply a list of 'feature' objects")
           return(TRUE)
         },
         prototype(features=list(),
                   provider="",
                   name="",
                   misc=list()))

setMethod('show', 'featvec',
          function(object) {
            cat("Feature fingerprint\n")
            cat(" name = ", object@name, "\n")
            cat(" source = ", object@provider, "\n")
            cat(" features = ", paste(sapply(object@features, as.character), collapse=' '), "\n")
          })
setMethod('as.character', 'featvec', function(x) {
  return(paste(sapply(x@features, as.character), collapse=' '))
})
setMethod("length", "featvec", function(x) {
  length(x@features)
})

## featvec.to.binaryfp <- function(fps, bit.length = 256) {
##   if (!all(sapply(fps, class) == 'featvec'))
##     stop("Must supply a list of feature vector fingerprints")
##   ## get all the features
##   features <- sort(unique(unlist(lapply(fps, as.numeric))))
##   nbit <- length(features)
##   if (nbit %% 2 == 1) nbit <- nbit + 1
##   ## based on the entire feature set, convert original fps to binary fps
##   fps <- lapply(fps, function(x) {
##     bitpos <- match(as.numeric(x), features)
##     new("fingerprint", nbit=nbit, folded=FALSE, provider=x@provider,name=x@name, bits=bitpos)
##   })
##   return(fps)
## }

balance <- function(fplist) {
  if (is.list(fplist)) {
    lapply(fplist, function(fp) {
      compl <- !fp
      new('fingerprint',
          nbit = 2 * length(fp),
          bits = c(fp@bits, compl@bits+length(fp)),
          provider='R', name='balanced')
    })
  } else {
    fp <- fplist
    compl <- !fp
    new('fingerprint',
        nbit = 2 * length(fp),
        bits = c(fp@bits, compl@bits+length(fp)),
        provider='R', name='balanced')    
  }
}

bit.importance <- function(actives, background) {
  bs.actives <- bit.spectrum(actives)
  bs.background <- bit.spectrum(background)

  m <- length(actives)
  n <- length(background)
  pa <- (m*bs.actives+bs.background)/(m+1)
  pb <- (n*bs.background+bs.actives)/(n+1)

  kl <- pa * log(pa/pb) + (1-pa) * log( (1-pa)/(1-pb) )
  kl[is.nan(kl)] <- NA
  return(kl)
}

bit.spectrum <- function(fplist) {
  if (!is.list(fplist)) stop("Must provide a list of fingerprint objects")
  if (any(!unlist(lapply(fplist, inherits, "fingerprint"))))
    stop("Must provide a list of fingerprint objects");
  nbit <- length(fplist[[1]])
  spec <- numeric(nbit)
  for (i in 1:length(fplist)) {
    bits <- fplist[[i]]@bits
    spec[bits] <- spec[bits]+1
  }
  spec / length(fplist)
}

shannon <- function(fplist) {
  if (!is.list(fplist)) stop("Must provide a list of fingerprint objects")
  if (any(!unlist(lapply(fplist, inherits, "fingerprint"))))
    stop("Must provide a list of fingerprint objects");

  bs <- bit.spectrum(fplist)
  bs <- bs[ bs != 0 ]
  -1 * sum( bs * log2(bs) )
}

\name{bit.spectrum}
\alias{bit.spectrum}
\title{
	Generate a Bit Spectrum from a List of Fingerprints
}
\description{
The idea of comparing datasets using fingerprints was described in
Guha & Schurer (2008). The idea is that one can summarize the dataset
by counting the frequency of occurrence of each bit position. The
frequency is normalized by the number of fingerprints considered. Thus a
collection of N fingerprints can be converted to a single vector of
numbers highlighting the most frequent bits with respect to a given
dataset. A plot of this vector looks like a traditional spectrum and
hence the name.

The bit spectra for two datasets (assuming that the same types of
fingerprints have been used) allows one to compare the similarity of
the datasets, without having to do a full pairwise similarity
calculation. The difference between the structural features of the
datasets can be quantified by evaluating the distance between the two
bit spectra.
}
\usage{
bit.spectrum(fplist)
}
\arguments{
  \item{fplist}{
        A list structure with each element being an object of class
	\code{fingerprint}. These will can be constructed by hand or
	read from disk via \code{\link{fp.read}}.

	All fingerprints in the list should be of the same length.
    }
}
\value{
A numeric vector of length equal to the size of the fingerprints.
}
\seealso{
    \code{\link{distance}}, \code{\link{fp.read}}
}
\references{
Guha, R.; Schurer, S.; \emph{J. Comp. Aid. Molec. Des.}, \bold{2008},
    \emph{22}, 367-384.
}
\keyword{programming}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}

\name{as.character}
\alias{as.character}
\alias{as.character,fingerprint-method}
\alias{as.character,featvec-method}
\alias{as.character,feature-method}
\title{
  Generates a String Representation of a Fingerprint
}
\description{
    The function returns a string of 1's and 0's or a character vector of
    features depending on the nature of the fingerprint supplied.        
}
\usage{
\S4method{as.character}{fingerprint}(x)
\S4method{as.character}{featvec}(x)
\S4method{as.character}{feature}(x)
}
\arguments{
    \item{x}{
      An object of class \code{fingerprint}, \code{featvec} or \code{feature}
    }
}
\value{
A string of 1's and 0's or else a character vector of features (with their counts)
}

\examples{
# make a fingerprint vector
fp <- new("fingerprint", nbit=32, bits=sample(1:32, 20))

# print out the string representation
as.character(fp)
}
\keyword{logic}
\keyword{methods}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}

\name{random.fingerprint}
\alias{random.fingerprint}
\title{
  Generate Randomized Fingerprints
}
\description{
  A utility function that can be used to generate binary fingerprints
  of a specified length with a specifed number of bit positions
  (selected randomly) set to 1. Currently bit positions are selected uniformly
}
\usage{
random.fingerprint(nbit,on)
}
\arguments{
    \item{nbit}{
      The length of the fingerprint, that is, the total number of bits.
      Must be a positive integer.
    }
    \item{on}{
      How many positions should be set to 1
      }
}
\value{
An object of class \code{fingerprint}
}
\examples{
# make a fingerprint vector
fp <- random.fingerprint(32, 16)
as.character(fp)
}
\keyword{logic}
\author{Rajarshi Guha \email{rguha@indiana.edu}}

\name{balance}
\alias{balance}
\title{
Generate a Balanced Code Fingerprint
}
\description{
It has been noted that the bit density in a fingerprint can affect its ability to
retrieve similar compounds from a database primarily due to complexity effects. One
approach to alleviating these effects is to generate fingerprints that have a bit
density of 50% (i.e., half the bits are set to 1). This method implements the 
balanced code approach described by Nisius and Bajorath to convert an ordinary
binary fingerprint (whose bit density is not 50%) to one that has a bit density of
50%. This is acheived by appending the complement of the input fingerprint to itself
(resulting in a fingerprint twice the size of the original).
}
\usage{
balance(fplist)
}
\arguments{
\item{fplist}{A single fingerprint or a list of fingerprints}
}
\value{
A single fingerprint objects or list of fingerprint objects that are "balanced",
in that they have a bit density of 50%. Their size is 2x the size of the input
fingerprints.
}
\seealso{
\code{\link{bit.spectrum}}, \code{\link{bit.importance}}
}
\references{
Nisius, B.; Bajorath, J.; \emph{ChemMedChem}, \bold{2010},
\emph{5}, 859-868.
}
\keyword{programming}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}

\name{fp.sim.matrix}
\alias{fp.sim.matrix}
\title{
  Calculates a Similarity Matrix for a Set of Fingerprints
}
\description{
Given a set of fingerprints, a pairwise similarity can be calculated using the
various distance metrics defined for binary strings. This function calculates
the pairwise similarity matrix for a set of \code{fingerprint} or
\code{featvec} objects supplied in a \code{list} 
structure. Any of the distance metrics provided by \code{\link{distance}} can be used and the 
default is the Tanimoto metric.

Note that if the the Euclidean distance is specified then the resultant matrix is a
distance matrix and not a similarity matrix
}
\usage{
fp.sim.matrix(fplist, fplist2=NULL, method='tanimoto')
}
\arguments{
  \item{fplist}{
        A list structure with each element being an object of class
	\code{fingerprint} or \code{featvec}. These can be constructed by hand or
	read from disk via \code{\link{fp.read}}    
    }
\item{fplist2}{A list structure with each element being an object of class
\code{fingerprint} or \code{featvec}. if \code{NULL} then traditional pairwise 
similarity is calculated with each member in \code{fplist}, otherwise the 
resultant N x M matrix is derived from the similarity between each member of 
\code{fplist} and \code{fplist2}}
    \item{method}{
    The type of distance metric to use. The default is \code{tanimoto}. Partial
    matching is supported.
    }
}
\value{
A matrix with dimensions equal to \code{(length(fplist), length(fplist))} if 
\code{fplist2} is NULL, otherwise \code{(length(fplist), length(fplist2))}
}
\seealso{
    \code{\link{distance}}, \code{\link{fp.read}}
}
\examples{
# make fingerprint objects
fp1 <- new("fingerprint", nbit=6, bits=c(1,2,5,6))
fp2 <- new("fingerprint", nbit=6, bits=c(1,4,5,6))
fp3 <- new("fingerprint", nbit=6, bits=c(2,3,4,5,6))

fp.sim.matrix( list(fp1,fp2,fp3) )
}
\keyword{logic}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}

\name{fplogical}
\alias{!}
\alias{|}
\alias{&}
\alias{xor}
\alias{|,fingerprint,fingerprint-method}
\alias{&,fingerprint,fingerprint-method}
\alias{xor,fingerprint,fingerprint-method}
\alias{!,fingerprint-method}
\title{
    Logical Operators for Fingerprints
}
\description{
These functions perform logical operatiosn (AND, OR, NOT, XOR) on the supplied
binary fingerprints. Thus for two fingerprints A and B we have
\describe{
\item{\code{&}}{Logical AND}
\item{\code{|}}{Logical OR}
\item{\code{xor}}{Logical XOR}
\item{\code{!}}{Logical NOT (negation)}
}
}
\arguments{
    \item{e1}{
      An object of class \code{fingerprint}
    }
    \item{e2}{
      An object of class \code{fingerprint}
    }
  }
\value{
A fingerprint object
}
\keyword{logic}
\keyword{methods}
\author{Rajarshi Guha \email{rguha@indiana.edu}}

\name{show}
\alias{show,fingerprint-method}
\alias{show,featvec-method}
\alias{show,feature-method}
\title{
    String Representation of a Fingerprint or Feature
}
\description{
Simply summarize the fingerprint or feature
}

\usage{
\S4method{show}{fingerprint}(object)
\S4method{show}{featvec}(object)
\S4method{show}{feature}(object)
}
\arguments{
    \item{object}{
      An object of class \code{fingerprint}, \code{featvec} or \code{feature}
    }
}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}
\keyword{logic}

\name{shannon}
\alias{shannon}
\alias{entropy}
\title{
Evaluate Shannon Entropy for a Set of Fingerprints
}
\description{
This method evaluates the Shannon entropy for a set of fingerprints
and utilizes the \code{\link{bit.spectrum}} method to obtain the relative
frequencies of individual bits
}
\usage{
shannon(fplist)
}
\arguments{
  \item{fplist}{
        A list structure with each element being an object of class
	\code{fingerprint}. These will can be constructed by hand or
	read from disk via \code{\link{fp.read}}.

	All fingerprints in the list should be of the same length.
    }
}
\value{
The Shannon entropy for the set of fingerprints
}
\seealso{
\code{\link{bit.spectrum}}, \code{\link{fp.read}}
}
\keyword{programming}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}

\name{bit.importance}
\alias{bit.importance}
\title{
Evaluate the Discriminatory Power of Individual Bits in a Binary Fingerprint
}
\description{
This method evaluates the Kullback-Leibler (KL) divergence to rank the 
individual bits in a binary fingerprint in their ability to discriminate 
between database and active compounds. This method is implemented based on
Nisius and Bajorath and includes an m-estimate correction.
}
\usage{
bit.importance(actives, background)
}
\arguments{
\item{actives}{A list of fingerprints for the actives}
\item{background}{A list of fingerprints representing the background collection}
}
\value{
A numeric vector of length equal to the size of the fingerprints. Each element
of the vector is the KL divergence for the corresponding bit. If a bit position
is never set to 1 in any of the compounds from the actives and the background, then
the KL divergence for that position is undefined and \code{NA} is returned.
}
\seealso{
\code{\link{bit.spectrum}}
}
\references{
Nisius, B.; Bajorath, J.; \emph{ChemMedChem}, \bold{2010},
\emph{5}, 859-868.
}
\keyword{programming}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}

\name{distance-methods}
\docType{methods}
\alias{distance}
\alias{distance-methods}
\alias{distance,featvec,featvec,character,missing,missing-method}
\alias{distance,featvec,featvec,missing,missing,missing-method}
\alias{distance,fingerprint,fingerprint,character,missing,missing-method}
\alias{distance,fingerprint,fingerprint,character,numeric,numeric-method}
\alias{distance,fingerprint,fingerprint,missing,missing,missing-method}
\title{Calculates the Similarity or Dissimilarity Between Two Fingerprints}
\description{
  A number of distance metrics can be calculated for binary
  fingerprints. Some of these are actually similarity metrics and
  thus represent the reverse of a distance metric.

  The following are distance (dissimilarity) metrics
  \itemize{
    \item Hamming
    \item Mean Hamming
    \item Soergel
    \item Pattern Difference
    \item Variance
    \item Size
    \item Shape
  }

  The following metrics are similarity metrics and so the distance can
  be obtained by subtracting the value fom 1.0
  \itemize{
    \item Tanimoto
    \item Dice
    \item Modified Tanimoto
    \item Simple
    \item Jaccard
    \item Russel-Rao
    \item Rodgers Tanimoto
    \item Cosine
    \item Achiai
    \item Carbo
    \item Baroniurbanibuser
    \item Kulczynski2
    \item Robust
  } 

  Finally the method also provides a set of composite and asymmetric
  distance metrics
  \itemize{
    \item Hamann
    \item Yule
    \item Pearson
    \item Dispersion
    \item McConnaughey
    \item Stiles
    \item Simpson
    \item Petke
    \item Tversky
  }
  The default metric is the Tanimoto coefficient.
}

\section{Methods}{
\describe{

\item{\code{signature(fp1 = "featvec", fp2 = "featvec", method = "character", a = "missing", b = "missing")}}{
 Similarity method for feature vector type fingerprints, supporting \code{tanimoto}, \code{robust} and \code{dice}
 metrics.
}

\item{\code{signature(fp1 = "featvec", fp2 = "featvec", method = "missing", a = "missing", b = "missing")}}{
 Evaluate Tanimoto similarity between two feature vector fingerprints			  
}

\item{\code{signature(fp1 = "fingerprint", fp2 = "fingerprint", method = "character", a = "missing", b = "missing")}}{
 Evaluate similarity (or dissimilrity) between two binary fingerprints. See below for a list of possible
 similarity (or dissimilarity) metrics			  
}

\item{\code{signature(fp1 = "fingerprint", fp2 = "fingerprint", method = "character", a = "numeric", b = "numeric")}}{
 Evaluate Tversky similarity between two binary fingerprints.
}

\item{\code{signature(fp1 = "fingerprint", fp2 = "fingerprint", method = "missing", a = "missing", b = "missing")}}{
 Evaluate Tanimoto similarity between two binary fingerprints
}
}}
\usage{
distance(fp1, fp2, method, a, b)
}
\arguments{
  \item{fp1}{
    An object of class \code{fingerprint} or \code{featvec}
  }
  \item{fp2}{
    An object of class \code{fingerprint} or \code{featvec}
  }
  \item{a}{Parameter for the Tversky index}
  \item{b}{Parameter for the Tversky index}
  \item{method}{
    The type of distance metric desired. Partial matching is
    supported and the deault is \code{tanimoto}. Alternative values are
    \itemize{
      \item \code{euclidean} 
      \item \code{hamming}
      \item \code{meanHamming}
      \item \code{soergel}
      \item \code{patternDifference}
      \item \code{variance}
      \item \code{size}
      \item \code{shape}

      \item \code{jaccard}
      \item \code{dice}
      \item \code{mt}
      \item \code{simple}
      \item \code{russelrao}
      \item \code{rodgerstanimoto}
      \item \code{cosine}
      \item \code{achiai}
      \item \code{carbo}
      \item \code{baroniurbanibuser}
      \item \code{kulczynski2}
      \item \code{robust}

      \item \code{hamann}
      \item \code{yule}
      \item \code{pearson}
      \item \code{mcconnaughey}
      \item \code{stiles}

      \item \code{simpson}
      \item \code{petke}
      \item \code{tversky}
    }
    If the two fingerprints are of class \code{featvec} then the following methods 
    may be specified: \code{tanimoto}, \code{robust} and \code{dice}.

  }
}
\value{
  Numeric value representing the distance in the specified metric between the
  supplied fingerprint objects
}
\examples{
# make a 2 fingerprint vectors
fp1 <- new("fingerprint", nbit=6, bits=c(1,2,5,6))
fp2 <- new("fingerprint", nbit=6, bits=c(1,2,5,6))

# calculate the tanimoto coefficient
distance(fp1,fp2) # should be 1

# Invert the second fingerprint
fp3 <- !fp2

distance(fp1,fp3) # should be 0
}

\references{Fligner, M.A.; Verducci, J.S.; Blower, P.E.;
  A Modification of the Jaccard-Tanimoto Similarity Index for
  Diverse Selection of Chemical Compounds Using Binary Strings,
  \emph{Technometrics}, 2002, \emph{44}(2), 110-119

  Monve, V.; Introduction to Similarity Searching in
  Chemistry, \emph{MATCH - Comm. Math. Comp. Chem.}, 2004, \emph{51}, 7-38
}
\keyword{logic}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}

\name{featvec-class}
\docType{class}
\alias{featvec-class}
\alias{distance,featvec,featvec,missing-method}
\alias{distance,featvec,featvec,character-method}
\alias{length,featvec-method}

\title{Class "featvec"}
\description{This class represents feature vector style fingerprints, where, rather than
a bit string, the fingerprint is represented as a sequence of (signed) integers or strings.
Each element of the collection is a representation of a structural feature. For cases where the
features are integers, this usually corresponds to a hash of the original feature string.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("featvec", ...)}.
In contrast to traditional binary fingerprints, operations on feature vectors
are slightly different and essentially correspond to operations on sets. Thus 
the logical and (&) would correspond to the union of the two feature vectors.
}
\section{Slots}{
	 \describe{
    \item{\code{features}:}{Object of class \code{"character"} ~~ A vector
      containing the numeric or character features. Numeric features are treated
      as character strings }
    \item{\code{provider}:}{Object of class \code{"character"} ~~
      Indicates the source of the fingerprint. Can be useful to keep
      track of what software generated the fingerprint.}
    \item{\code{name}:}{Object of class \code{"character"} ~~
      The name associated with the fingerprint. If not name is available
    this gets set to an empty string}
    \item{\code{misc}:}{A list to hold arbitrary items associated with a fingerprint (such as 
     extra fields from a fingerprint file)}
  }
}
\section{Methods}{
  \describe{
    \item{distance}{\code{signature(fp1 = "featvec", fp2 = "featvec", method = "missing")}: ... }
    \item{distance}{\code{signature(fp1 = "featvec", fp2 = "featvec", method = "character")}: ... }
    \item{as.character}{\code{signature(fp = "featvec")}: ... }
    \item{length}{\code{signature(fp = "featvec")}: ... }
    \item{show}{\code{signature(fp = "featvec")}: ... }
}
}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}
\seealso{
  \code{\link{fp.read}}, \code{\link{fp.read.to.matrix}}
  \code{\link{fp.sim.matrix}}, \code{\link{fp.to.matrix}},
  \code{\link{fp.factor.matrix}}
  \code{\link{random.fingerprint}}
}

 
\keyword{classes}
\keyword{logic}

\name{count-methods}
\docType{methods}

\alias{count-methods}
\alias{count}
\alias{count,feature-method}

\alias{count<--methods}
\alias{count<-}
\alias{count<-,feature,numeric-method}

\title{Get or Set Count of Occurence of a Feature}
\description{
  Get or set the count of occurence associated with a
  \code{\link{feature-class}} object. The default value for the getter
  (as defined in the prototype) is 1.
}

\section{Methods}{
  \describe{
    \item{\code{signature(object = "feature")}}{Return the count associated with the
      feature object}
    \item{\code{signature(x = "feature", value = "numeric")}}{Set the count associated with the
      feature object}
  }
}

\usage{
\S4method{count}{feature}(object)
\S4method{count}{feature,numeric}(x) <- value
}
\arguments{
  \item{object}{
    An object of class \code{\link{feature-class}}
  }
  \item{x}{
    An object of class \code{\link{feature-class}}
  }
  \item{value}{
    A numeric (which will be coerced to \code{integer}) indicating the
    count associated with the feature
  }
}
\value{
  An integer representing count of occurence of the feature
}
\keyword{programming}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}

\name{length}
\alias{length}
\alias{length,fingerprint-method}
\title{
  Fingerprint Bit Length
}
\description{
  Returns the length of the fingerprint. That is, this is the length of
  the entire bit string and not simply the number of bits that are on.
}
\usage{
  \S4method{length}{fingerprint}(x)
}
\arguments{
    \item{x}{
      An object of class \code{fingerprint}
    }
}
\value{
The length of the bit string
}
\keyword{logic}
\keyword{methods}
\author{Rajarshi Guha \email{rguha@indiana.edu}}

\name{euc.vector}
\alias{euc.vector}
\title{
  Euclidean Representation of Binary Fingerprints
}
\description{
 Ordinarily, a binary fingerprint can be considered to represent a 
 corner of a nD hypercube. However in many cases using such a representation
 can lead to a very sparse space. Consequently one approach is to convert
 the fingerprint so that it represents points on a nD unit hypersphere.

 The resultant fingerprint is then a nD coordinate.
}
\usage{
euc.vector(fp)
}
\arguments{
    \item{fp}{
        An object of class \code{fingerprint}.
    }
}
\value{
A numeric of length equal to the bit length of the fingerprint. The
result corresponds to a unit vector for a point
on the nD hypersphere
}
\examples{
# make a fingerprint vector
fp <- new("fingerprint", nbit=8, bits=c(1,3,4,5,7))
vec <- euc.vector(fp)
}
\keyword{logic}
\author{Rajarshi Guha \email{rguha@indiana.edu}}

\name{fp.factor.matrix}
\alias{fp.factor.matrix}
\title{
  Converts a List of Fingerprints to a data.frame of Factors
}
\description{
This function will convert  a \code{list} of fingerprint objects
to a \code{data.frame} of factors with levels 1 and 0.
}
\usage{
fp.factor.matrix(fplist)
}
\arguments{
  \item{fplist}{
        A list structure with each element being an object of class
	\code{fingerprint}. These will can be constructed by hand or
	read from disk via \code{\link{fp.read}}    
    }
}
\value{
A matrix with dimensions equal to \code{(length(fplist), length(fplist))}
}
\seealso{
    \code{\link{distance}}, \code{\link{fp.read}}
}
\examples{
# make fingerprint objects
fp1 <- new("fingerprint", nbit=6, bits=c(1,2,5,6))
fp2 <- new("fingerprint", nbit=6, bits=c(1,4,5,6))
fp3 <- new("fingerprint", nbit=6, bits=c(2,3,4,5,6))

fp.factor.matrix( list(fp1,fp2,fp3) )
}
\keyword{logic}
\author{Rajarshi Guha \email{rguha@indiana.edu}}

\name{feature-class}
\docType{class}
\alias{feature-class}

\title{Class "feature"}
\description{This class represents features - arbitrary alphanumeric sequences
  that are used to characterize molecular substructures (though there is no real
  restriction to molecules). A feature is associated with an integer count, 
  indicating the occurence of that feature in a molecule. The default value is 1.
}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("feature", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{feature}:}{Object of class \code{"character"} ~ The string representation of a feature }
    \item{\code{count}:}{Object of class \code{"integer"} ~ The occurence of the feature. Default is 1}
    \item{\code{.Data}:}{???}
  }
}

\section{Methods}{
  \describe{
    \item{count}{\code{signature(object = "feature")}: Return the count associated with the
      feature}
  }
}

\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}
\seealso{
  \code{\link{featvec-class}}
}

\examples{
  ## create a new feature
  f <- new("feature", feature='ABCD', count=as.integer(1))

  ## modify the feature string and the count
  feature(f) <- 'UXYZ'
  count(f) <- 10
}
\keyword{classes}
\keyword{logic}

\name{fingerprint-class}
\docType{class}
\alias{fingerprint-class}

\alias{euc.vector,fingerprint-method}
\alias{fold,fingerprint-method}
\alias{random.fingerprint,numeric,numeric-method}

\title{Class "fingerpint"}
\description{This class represents binary fingerprints, usually
  generated by a variety of cheminformatics software, but not
  restricted to such
  }
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("fingerprint", ...)}.
	Fingerprints can traditionally thought of as a vector of 1's and
	0's. However for large fingerprints this is inefficient and
	instead we simply store the positions of the bits that are
	on. Certain operations also need to know the length of the
	original bit string and this length is stored in the object at
	construction. Even though we store extra information along with
	the bit positions, conceptually we still consider the objects as
	simple bit strings. Thus the usual bitwise logical operations
	(&, |, !, xor) can be applied to objects of this class.
}
\section{Slots}{
	 \describe{
    \item{\code{bits}:}{Object of class \code{"numeric"} ~~ A vector
      indicating the bit positions that are on. }
    \item{\code{nbit}:}{Object of class \code{"numeric"} ~~ Indicates the length of the original bit string.}
    \item{\code{folded}:}{Object of class \code{"logical"} ~~ Indicates
      whether the fingerprint has been folded.}
    \item{\code{provider}:}{Object of class \code{"character"} ~~
      Indicates the source of the fingerprint. Can be useful to keep
      track of what software generated the fingerprint.}
    \item{\code{name}:}{Object of class \code{"character"} ~~
      The name associated with the fingerprint. If not name is available
    this gets set to an empty string}
    \item{\code{misc}:}{Object of class \code{"list"} ~~
    A holder for arbitrary items that may have been stored along with the fingerprint. Only
    certain formats allow extra items to be stored with the fingerprint, so in many cases
    this field is just an empty list}

  }
}
\section{Methods}{
  \describe{
    \item{distance}{\code{signature(fp1 = "fingerprint", fp2 = "fingerprint", method = "missing", a = "missing", b = "missing")}: ... }
    \item{distance}{\code{signature(fp1 = "fingerprint", fp2 = "fingerprint", method = "character", a = "missing", b = "missing")}: ... }
    \item{euc.vector}{\code{signature(fp = "fingerprint")}: ... }
    \item{fold}{\code{signature(fp = "fingerprint")}: ... }
    \item{random.fingerprint}{\code{signature(nbit = "numeric", on = "numeric")}: ... }    
	 }
}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}
\seealso{
  \code{\link{fp.read}}, \code{\link{fp.read.to.matrix}}
  \code{\link{fp.sim.matrix}}, \code{\link{fp.to.matrix}},
  \code{\link{fp.factor.matrix}}
  \code{\link{random.fingerprint}}
  }

\examples{
## make fingerprints
x <- new("fingerprint", nbit=128, bits=sample(1:128, 100))
y <- x
distance(x,y) # should be 1
x <- new("fingerprint", nbit=128, bits=sample(1:128, 100))
distance(x,y)
folded <- fold(x)

## binary operations on fingerprints
x <- new("fingerprint", nbit=8, bits=c(1,2,3,6,8))
y <- new("fingerprint", nbit=8, bits=c(1,2,4,5,7,8))
x & y
x | y
!x
}
\keyword{classes}
\keyword{logic}

\name{feature-methods}
\docType{methods}

\alias{feature}
\alias{feature-methods}
\alias{feature,feature-method}

\alias{feature<--methods}
\alias{feature<-}
\alias{feature<-,feature,character-method}

\title{Get or Set the Character String Representing the Feature}
\description{
  Get or set the character string representing a feature of a 
  \code{\link{feature-class}} object. The default value for the getter
  (as defined in the prototype) is the empty string.
}

\section{Methods}{
  \describe{
    \item{\code{signature(object = "feature")}}{Return the feature associated with the
      feature object}
    \item{\code{signature(x = "feature", value = "character")}}{Set the feature associated with the
      feature object}
  }
}

\usage{
\S4method{feature}{feature}(object)
\S4method{feature}{feature,character}(x) <- value
}

\arguments{
  \item{object}{
    An object of class \code{\link{feature-class}}
  }
  \item{x}{
    An object of class \code{\link{feature-class}}
  }
  \item{value}{
    The character string to replace the current feature string with
  }
}
\value{
  An character string representing the feature
}
\keyword{programming}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}

\name{fp.to.matrix}
\alias{fp.to.matrix}
\title{
  Converts a List of Fingerprints to a Matrix
}
\description{
    In general, fingerprint data is read from a file or obtained via
    calls to an external generator and the return value is a list of fingerprints.
    This function takes the list and returns a matrix having number of rows equal to 
    the number of fingerprints and the number of columns equal to the length of
    the fingerprint. Each element is 1 or 0 (1's being specified by the positions
    in each fingerprint vector)
}
\usage{
fp.to.matrix(fplist)
}
\arguments{
    \item{fplist}{
        A list structure with each element being an object of class
	\code{fingerprint}. These will can be constructed by hand or
	read from disk via \code{\link{fp.read}}
    }
}
\value{
A matrix with dimensions equal to \code{length(fplist), bit length)}
where bit length is a property of the fingerprint objects in the list.
}
\seealso{
    \code{\link{distance}}, \code{\link{fp.read}}
}
\examples{
# make fingerprint objects
fp1 <- new("fingerprint", nbit=6, bits=c(1,2,5,6))
fp2 <- new("fingerprint", nbit=6, bits=c(1,4,5,6))
fp3 <- new("fingerprint", nbit=6, bits=c(2,3,4,5,6))

fp.to.matrix( list(fp1,fp2,fp3) )
}
\keyword{logic}
\author{Rajarshi Guha \email{rguha@indiana.edu}}

\name{c}
\alias{c,feature-method}
\title{
    Combine Multiple Features to Give a List of Features
}
\description{
  Combine multiple \code{feature} objects to give a list of feature objects
}

\usage{
\S4method{c}{feature}(x, ..., recursive = FALSE)
}
\arguments{
    \item{x}{
      An object of class \code{feature}
    }
    \item{...}{
      One or more \code{feature} objects
    }
    \item{recursive}{ Ignored }
}
\author{Rajarshi Guha \email{rajarshi.guha@gmail.com}}
\keyword{logic}

\name{fold}
\alias{fold}
\title{
  Fold a fingerprint
}
\description{
    In many situations a fingerprint is generated using a large length (such as 1024 bits or more).
    As a result of this, the fingerprints for a dataset can be very sparse. One approach to increasing 
    bit density of such fingerprints is to fold them. This is performed by dividing the original
    fingerprint bitstring into two substrings of equal length and then perform an OR on
    the two substrings. 

    It should be noted that many fingerprint generating routines will perform this internally.
}
\usage{
fold(fp)
}
\arguments{
    \item{fp}{
        The fingerprint to fold. Should be of class \code{fingerprint}.
    }
}
\value{
An object of class \code{fingerprint} representing the folded fingerprint.
}
\examples{
# make a fingerprint vector
fp <- new("fingerprint", nbit=64, bits=sample(1:64, 30))
fold(fp)
}
\keyword{logic}
\author{Rajarshi Guha \email{rguha@indiana.edu}}
